@@ -1,36 +1,24 @@
 /* TN5250
  * Copyright (C) 1997 Michael Madore
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * This file is part of TN5250.
+ *
+ * TN5250 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1, or (at your option)
  * any later version.
  * 
- * This program is distributed in the hope that it will be useful,
+ * TN5250 is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this software; see the file COPYING.  If not, write to
  * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307 USA
  * 
- * As a special exception, the copyright holder gives permission
- * for additional uses of the text contained in its release of TN5250.
- * 
- * The exception is that, if you link the TN5250 library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the TN5250 library code into it.
- * 
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- * 
- * If you write modifications of your own for TN5250, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice. */
+ */
 #define _TN5250_TERMINAL_PRIVATE_DEFINED
 #include "tn5250-private.h"
 
@@ -41,50 +29,43 @@
 #define A_VERTICAL ((1UL) << ((22) + 8))
 #endif				/* A_VERTICAL */
 
+/* Some older versions of ncurses don't define NCURSES_COLOR_T */
+#ifndef NCURSES_COLOR_T
+#define NCURSES_COLOR_T short
+#endif
+
 /* Mapping of 5250 colors to curses colors */
-#define A_5250_GREEN    ((attr_t)COLOR_PAIR(COLOR_GREEN))
-#define A_5250_WHITE    ((attr_t)COLOR_PAIR(COLOR_WHITE) | A_BOLD)
-#define A_5250_RED      ((attr_t)COLOR_PAIR(COLOR_RED))
-#define A_5250_TURQ     ((attr_t)COLOR_PAIR(COLOR_CYAN))
-#define A_5250_YELLOW   ((attr_t)COLOR_PAIR(COLOR_YELLOW) | A_BOLD)
-#define A_5250_PINK     ((attr_t)COLOR_PAIR(COLOR_MAGENTA))
-#define A_5250_BLUE     ((attr_t)COLOR_PAIR(COLOR_CYAN) | A_BOLD)
+struct _curses_color_map {
+   char *name;
+   NCURSES_COLOR_T ref;
+   attr_t bld;
+};
+typedef struct _curses_color_map curses_color_map;
+
+static curses_color_map colorlist[] =
+{
+  { "black",     COLOR_BLACK         },
+  { "red",       COLOR_RED,   A_BOLD },
+  { "green",     COLOR_GREEN         },
+  { "yellow",    COLOR_YELLOW,A_BOLD },
+  { "blue",      COLOR_CYAN,  A_BOLD },
+  { "pink",      COLOR_MAGENTA       },
+  { "turquoise", COLOR_CYAN          },
+  { "white",     COLOR_WHITE, A_BOLD },
+  { NULL, -1 }
+};
+
+#define A_5250_GREEN    ((attr_t)COLOR_PAIR(COLOR_GREEN)|colorlist[COLOR_GREEN].bld)
+#define A_5250_WHITE    ((attr_t)COLOR_PAIR(COLOR_WHITE)|colorlist[COLOR_WHITE].bld)
+#define A_5250_RED      ((attr_t)COLOR_PAIR(COLOR_RED)|colorlist[COLOR_RED].bld)
+#define A_5250_TURQ     ((attr_t)COLOR_PAIR(COLOR_CYAN)|colorlist[COLOR_CYAN].bld)
+#define A_5250_YELLOW   ((attr_t)COLOR_PAIR(COLOR_YELLOW)|colorlist[COLOR_YELLOW].bld)
+#define A_5250_PINK     ((attr_t)COLOR_PAIR(COLOR_MAGENTA)|colorlist[COLOR_MAGENTA].bld)
+#define A_5250_BLUE     ((attr_t)COLOR_PAIR(COLOR_BLUE)|colorlist[COLOR_BLUE].bld)
 
 /*@-globstate -nullpass@*/  /* lclint incorrectly assumes stdscr may be NULL */
 
-static attr_t attribute_map[] =
-{A_5250_GREEN,
- A_5250_GREEN | A_REVERSE,
- A_5250_WHITE,
- A_5250_WHITE | A_REVERSE,
- A_5250_GREEN | A_UNDERLINE,
- A_5250_GREEN | A_UNDERLINE | A_REVERSE,
- A_5250_WHITE | A_UNDERLINE,
- 0x00,
- A_5250_RED,
- A_5250_RED | A_REVERSE,
- A_5250_RED | A_BLINK,
- A_5250_RED | A_BLINK | A_REVERSE,
- A_5250_RED | A_UNDERLINE,
- A_5250_RED | A_UNDERLINE | A_REVERSE,
- A_5250_RED | A_UNDERLINE | A_BLINK,
- 0x00,
- A_5250_TURQ | A_VERTICAL,
- A_5250_TURQ | A_VERTICAL | A_REVERSE,
- A_5250_YELLOW | A_VERTICAL,
- A_5250_YELLOW | A_VERTICAL | A_REVERSE,
- A_5250_TURQ | A_UNDERLINE | A_VERTICAL,
- A_5250_TURQ | A_UNDERLINE | A_REVERSE | A_VERTICAL,
- A_5250_YELLOW | A_UNDERLINE | A_VERTICAL,
- 0x00,
- A_5250_PINK,
- A_5250_PINK | A_REVERSE,
- A_5250_BLUE,
- A_5250_BLUE | A_REVERSE,
- A_5250_PINK | A_UNDERLINE,
- A_5250_PINK | A_UNDERLINE | A_REVERSE,
- A_5250_BLUE | A_UNDERLINE,
- 0x00};
+static attr_t attribute_map[33];
 
 static void curses_terminal_init(Tn5250Terminal * This) /*@modifies This@*/;
 static void curses_terminal_term(Tn5250Terminal * This) /*@modifies This@*/;
@@ -101,6 +82,7 @@ static int curses_terminal_getkey(Tn5250Terminal * This) /*@modifies This@*/;
 static int curses_terminal_get_esc_key(Tn5250Terminal * This, int is_esc) /*@modifies This@*/;
 static void curses_terminal_beep(Tn5250Terminal * This);
 static int curses_terminal_is_ruler(Tn5250Terminal *This, Tn5250Display *display, int x, int y);
+int curses_rgb_to_color(int r, int g, int b, int *rclr, int *rbold);
 
 
 #ifdef USE_OWN_KEY_PARSING
@@ -248,6 +230,9 @@ static Key curses_vt100[] = {
    { K_NEWLINE,         "\033\012" }, /* ESC ^J */
    { K_NEWLINE,         "\033\015" }, /* ESC ^M */
    { K_INSERT,		"\033\177" }, /* ESC DEL */
+   { K_NEXTWORD,	"\033\025" }, /* ESC Ctrl-U */
+   { K_PREVWORD,	"\033\010" }, /* ESC Ctrl-H */
+   { K_FIELDHOME,	"\033\006" }, /* ESC Ctrl-F */
    /* K_INSERT = ESC+DEL handled in code below. */
 };
 #endif
@@ -322,6 +307,7 @@ static void curses_terminal_init(Tn5250Terminal * This)
    int i = 0, c, s;
    struct timeval tv;
    char *str;
+   int x;
 
    (void)initscr();
    raw();
@@ -346,15 +332,58 @@ static void curses_terminal_init(Tn5250Terminal * This)
    /* Initialize colors if the terminal supports it. */
    if (has_colors()) {
       start_color();
-      init_pair(COLOR_BLACK, COLOR_BLACK, COLOR_BLACK);
-      init_pair(COLOR_GREEN, COLOR_GREEN, COLOR_BLACK);
-      init_pair(COLOR_RED, COLOR_RED, COLOR_BLACK);
-      init_pair(COLOR_CYAN, COLOR_CYAN, COLOR_BLACK);
-      init_pair(COLOR_WHITE, COLOR_WHITE, COLOR_BLACK);
-      init_pair(COLOR_MAGENTA, COLOR_MAGENTA, COLOR_BLACK);
-      init_pair(COLOR_BLUE, COLOR_BLUE, COLOR_BLACK);
-      init_pair(COLOR_YELLOW, COLOR_YELLOW, COLOR_BLACK);
+      init_pair(COLOR_BLACK, colorlist[COLOR_BLACK].ref, 
+                             colorlist[COLOR_BLACK].ref);
+      init_pair(COLOR_GREEN, colorlist[COLOR_GREEN].ref, 
+                             colorlist[COLOR_BLACK].ref);
+      init_pair(COLOR_RED,   colorlist[COLOR_RED  ].ref, 
+                             colorlist[COLOR_BLACK].ref);
+      init_pair(COLOR_CYAN,  colorlist[COLOR_CYAN ].ref, 
+                             colorlist[COLOR_BLACK].ref);
+      init_pair(COLOR_WHITE, colorlist[COLOR_WHITE].ref, 
+                             colorlist[COLOR_BLACK].ref);
+      init_pair(COLOR_MAGENTA,colorlist[COLOR_MAGENTA].ref, 
+                             colorlist[COLOR_BLACK].ref);
+      init_pair(COLOR_BLUE  ,colorlist[COLOR_BLUE ].ref, 
+                             colorlist[COLOR_BLACK].ref);
+      init_pair(COLOR_YELLOW,colorlist[COLOR_YELLOW].ref, 
+                             colorlist[COLOR_BLACK].ref);
    }
+
+   x=-1;
+   attribute_map[++x] = A_5250_GREEN;
+   attribute_map[++x] = A_5250_GREEN | A_REVERSE;
+   attribute_map[++x] = A_5250_WHITE;
+   attribute_map[++x] = A_5250_WHITE | A_REVERSE;
+   attribute_map[++x] = A_5250_GREEN | A_UNDERLINE;
+   attribute_map[++x] = A_5250_GREEN | A_UNDERLINE | A_REVERSE;
+   attribute_map[++x] = A_5250_WHITE | A_UNDERLINE;
+   attribute_map[++x] = 0x00;
+   attribute_map[++x] = A_5250_RED;
+   attribute_map[++x] = A_5250_RED | A_REVERSE;
+   attribute_map[++x] = A_5250_RED | A_BLINK;
+   attribute_map[++x] = A_5250_RED | A_BLINK | A_REVERSE;
+   attribute_map[++x] = A_5250_RED | A_UNDERLINE;
+   attribute_map[++x] = A_5250_RED | A_UNDERLINE | A_REVERSE;
+   attribute_map[++x] = A_5250_RED | A_UNDERLINE | A_BLINK;
+   attribute_map[++x] = 0x00;
+   attribute_map[++x] = A_5250_TURQ | A_VERTICAL;
+   attribute_map[++x] = A_5250_TURQ | A_VERTICAL | A_REVERSE;
+   attribute_map[++x] = A_5250_YELLOW | A_VERTICAL;
+   attribute_map[++x] = A_5250_YELLOW | A_VERTICAL | A_REVERSE;
+   attribute_map[++x] = A_5250_TURQ | A_UNDERLINE | A_VERTICAL;
+   attribute_map[++x] = A_5250_TURQ | A_UNDERLINE | A_REVERSE | A_VERTICAL;
+   attribute_map[++x] = A_5250_YELLOW | A_UNDERLINE | A_VERTICAL;
+   attribute_map[++x] = 0x00;
+   attribute_map[++x] = A_5250_PINK;
+   attribute_map[++x] = A_5250_PINK | A_REVERSE;
+   attribute_map[++x] = A_5250_BLUE;
+   attribute_map[++x] = A_5250_BLUE | A_REVERSE;
+   attribute_map[++x] = A_5250_PINK | A_UNDERLINE;
+   attribute_map[++x] = A_5250_PINK | A_UNDERLINE | A_REVERSE;
+   attribute_map[++x] = A_5250_BLUE | A_UNDERLINE;
+   attribute_map[++x] = 0x00;
+
    This->data->quit_flag = 0;
 
    /* Determine if the terminal supports underlining. */
@@ -616,6 +645,17 @@ static void curses_terminal_update(Tn5250Terminal * This, Tn5250Display *display
       }
       This->data->last_width = tn5250_display_width(display);
       This->data->last_height = tn5250_display_height(display);
+
+/* XXX: this is somewhat of a hack.  For some reason the change to
+      132 col lags a bit, causing our update to fail, so this just waits
+      for the update to finish...  (is there a better way?) */
+ 
+      for (x=0; x<10; x++) {
+          refresh ();
+          if (tn5250_display_width(display)==curses_terminal_width(This)-1)
+              break;
+          usleep(10000);
+      }
    }
    attrset(A_NORMAL);
    getmaxyx(stdscr, my, mx);
@@ -1019,6 +1059,16 @@ static int curses_terminal_get_esc_key(Tn5250Terminal * This, int is_esc)
       key = K_F24;
       break;
 
+   case K_CTRL('U'):
+      key = K_NEXTWORD;
+      break;
+   case K_CTRL('H'):
+      key = K_PREVWORD;
+      break;
+   case K_CTRL('F'):
+      key = K_FIELDHOME;
+      break;
+
       /* AS/400 strangeness */
    case 'A':
       key = K_ATTENTION;
@@ -1183,6 +1233,127 @@ static int curses_terminal_getkey (Tn5250Terminal *This)
 }
 #endif
 
+/****i* lib5250/curses_rgb_to_color
+ * NAME
+ *    curses_rgb_to_color
+ * SYNOPSIS
+ *    curses_rgb_to_color (r, g, b, &clr, &bld);
+ * INPUTS
+ *    int                  r          -    
+ *    int                  g          -    
+ *    int                  b          -    
+ *    int            *     rclr       -    
+ *    int            *     rbold      -    
+ * DESCRIPTION
+ *    DOCUMENT ME!!!
+ *****/
+int curses_rgb_to_color(int r, int g, int b, int *rclr, int *rbold) {
+
+    int clr;
+    
+    clr = ((r & 0xff) << 16) | ((g & 0xff) << 8) | (b & 0xff);
+    *rbold = A_NORMAL;
+
+    switch (clr) {
+       case 0xFFFFFF:
+	 *rclr = COLOR_WHITE;   *rbold = A_BOLD;
+         break;
+       case 0xFFFF00:
+	 *rclr = COLOR_YELLOW;  *rbold = A_BOLD;
+         break;
+       case 0xFF00FF:
+	 *rclr = COLOR_MAGENTA; *rbold = A_BOLD;
+         break;
+       case 0xFF0000:
+	 *rclr = COLOR_RED;     *rbold = A_BOLD;
+         break;
+       case 0x00FFFF:
+	 *rclr = COLOR_CYAN;    *rbold = A_BOLD;
+         break;
+       case 0x00FF00:
+	 *rclr = COLOR_GREEN;   *rbold = A_BOLD;
+         break;
+       case 0x0000FF:
+	 *rclr = COLOR_BLUE;    *rbold = A_BOLD;
+         break;
+       case 0x808080:
+	 *rclr = COLOR_WHITE;
+         break;
+       case 0xC0C0C0:
+	 *rclr = COLOR_WHITE;
+         break;
+       case 0x808000:
+	 *rclr = COLOR_YELLOW;
+         break;
+       case 0x800080:
+	 *rclr = COLOR_MAGENTA;
+         break;
+       case 0x800000:
+	 *rclr = COLOR_RED;
+         break;
+       case 0x008080:
+	 *rclr = COLOR_CYAN;
+         break;
+       case 0x008000:
+	 *rclr = COLOR_GREEN;
+         break;
+       case 0x000080:
+	 *rclr = COLOR_BLUE;
+         break;
+       case 0x000000:
+	 *rclr = COLOR_BLACK;
+         break;
+       default:
+         return -1;
+    }
+
+    return 0;
+}
+
+/****i* lib5250/tn5250_curses_terminal_load_colorlist
+ * NAME
+ *    tn5250_curses_terminal_load_colorlist
+ * SYNOPSIS
+ *    tn5250_curses_terminal_load_colorlist(config);
+ * INPUTS
+ *    Tn5250Config   *     config     -    
+ * DESCRIPTION
+ *    DOCUMENT ME!!!
+ *****/
+void tn5250_curses_terminal_load_colorlist(Tn5250Config *config) {
+
+   int r, g, b, x, clr, bld;
+
+   if (tn5250_config_get_bool(config, "black_on_white")) {
+       for (x=COLOR_BLACK+1; x<=COLOR_WHITE; x++) {
+           colorlist[x].ref = COLOR_BLACK;
+           colorlist[x].bld = A_NORMAL;
+       }
+       colorlist[COLOR_BLACK].ref = COLOR_WHITE;
+       colorlist[COLOR_BLACK].bld = A_BOLD;
+   }
+
+   if (tn5250_config_get_bool(config, "white_on_black")) {
+       for (x=COLOR_BLACK+1; x<=COLOR_WHITE; x++) {
+           colorlist[x].ref = COLOR_WHITE;
+           colorlist[x].bld = A_BOLD;
+       }
+       colorlist[COLOR_BLACK].ref = COLOR_BLACK;
+       colorlist[COLOR_BLACK].bld = A_NORMAL;
+   }
+
+   x=0;
+   while (colorlist[x].name != NULL) {
+       if (tn5250_parse_color(config, colorlist[x].name, &r, &g, &b)!=-1) {
+           if (curses_rgb_to_color(r,g,b, &clr, &bld) != -1) {
+                colorlist[x].ref = clr;
+                colorlist[x].bld = bld;
+           }
+       }
+       x++;
+   }
+
+}
 
 #endif /* USE_CURSES */
 
