@@ -1,36 +1,24 @@
 /* TN5250
  * Copyright (C) 1997 Michael Madore
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * This file is part of TN5250.
+ *
+ * TN5250 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1, or (at your option)
  * any later version.
  * 
- * This program is distributed in the hope that it will be useful,
+ * TN5250 is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this software; see the file COPYING.  If not, write to
  * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307 USA
  * 
- * As a special exception, the copyright holder gives permission
- * for additional uses of the text contained in its release of TN5250.
- * 
- * The exception is that, if you link the TN5250 library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the TN5250 library code into it.
- * 
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- * 
- * If you write modifications of your own for TN5250, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice. */
+ */
 #include "tn5250-private.h"
 
 #ifdef NDEBUG
@@ -710,4 +698,75 @@ int tn5250_dbuffer_msg_line (Tn5250DBuffer *This)
    return l;
 }
 
+/****f* lib5250/tn5250_dbuffer_prevword
+ * NAME
+ *    tn5250_dbuffer_prevword
+ * SYNOPSIS
+ *    tn5250_dbuffer_prevword (This);
+ * INPUTS
+ *    Tn5250DBuffer *      This       -
+ * DESCRIPTION
+ *    DOCUMENT ME!!!
+ *****/
+void tn5250_dbuffer_prevword(Tn5250DBuffer * This)
+{
+   int foundblank=0;
+   int state=0;
+   int maxiter;
+
+   TN5250_LOG (("dbuffer_prevword: entered.\n"));
+
+   maxiter = (This->w * This->h);
+   TN5250_ASSERT(maxiter>0);
+
+   while (--maxiter) {
+        tn5250_dbuffer_left(This);
+        switch (state) {
+           case 0:
+              if (This->data[This->cy * This->w + This->cx] <= 0x40) state++;
+              break;
+           case 1:
+              if (This->data[This->cy * This->w + This->cx] > 0x40) state++;
+              break;
+           case 2:
+              if (This->data[This->cy * This->w + This->cx] <= 0x40) {
+                   tn5250_dbuffer_right(This, 1);
+                   return;
+              }
+              break;
+        }
+   }
+ 
+}
+
+/****f* lib5250/tn5250_dbuffer_nextword
+ * NAME
+ *    tn5250_dbuffer_nextword
+ * SYNOPSIS
+ *    tn5250_dbuffer_nextword (This);
+ * INPUTS
+ *    Tn5250DBuffer *      This       -
+ * DESCRIPTION
+ *    DOCUMENT ME!!!
+ *****/
+void tn5250_dbuffer_nextword(Tn5250DBuffer * This)
+{
+   int foundblank=0;
+   int maxiter;
+
+   TN5250_LOG (("dbuffer_nextword: entered.\n"));
+
+   maxiter = (This->w * This->h);
+   TN5250_ASSERT(maxiter>0);
+
+   while (--maxiter) {
+      tn5250_dbuffer_right(This, 1);
+      if (This->data[This->cy * This->w + This->cx] <= 0x40) foundblank++;
+      if ((foundblank) && (This->data[This->cy * This->w + This->cx] > 0x40)) {
+           break;
+      }
+   }
+
+   ASSERT_VALID(This);
+}
 
