@@ -1,36 +1,24 @@
 /* TN5250
  * Copyright (C) 1997 Michael Madore
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * This file is part of TN5250.
+ *
+ * TN5250 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1, or (at your option)
  * any later version.
  * 
- * This program is distributed in the hope that it will be useful,
+ * TN5250 is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this software; see the file COPYING.  If not, write to
  * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307 USA
  * 
- * As a special exception, the copyright holder gives permission
- * for additional uses of the text contained in its release of TN5250.
- * 
- * The exception is that, if you link the TN5250 library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the TN5250 library code into it.
- * 
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- * 
- * If you write modifications of your own for TN5250, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice. */
+ */
 
 #include "tn5250-private.h"
 
@@ -59,6 +47,7 @@ Tn5250Display *tn5250_display_new()
    This->indicators = 0;
    This->indicators_dirty = 0;
    This->pending_insert = 0;
+   This->sign_key_hack = 1;
    This->session = NULL;
    This->key_queue_head = This->key_queue_tail = 0;
    This->saved_msg_line = NULL;
@@ -120,6 +109,10 @@ int tn5250_display_config(Tn5250Display *This, Tn5250Config *config)
       tn5250_config_unref (This->config);
    This->config = config;
 
+   /* check if the +/- sign keyboard hack should be enabled */
+   if(tn5250_config_get(config, "sign_key_hack")) 
+      This->sign_key_hack = tn5250_config_get_bool(config, "sign_key_hack");
+
    /* Set a terminal type if necessary */
    termtype = tn5250_config_get(config, "env.TERM");
 
@@ -260,7 +253,7 @@ void tn5250_display_set_terminal(Tn5250Display * This, Tn5250Terminal * term)
    This->terminal = term;
    This->indicators_dirty = 1;
    tn5250_terminal_update(This->terminal, This);
-   tn5250_terminal_update_indicators(This->terminal, This);
+/*   tn5250_terminal_update_indicators(This->terminal, This); */
 }
 
 /****f* lib5250/tn5250_display_update
@@ -278,7 +271,7 @@ void tn5250_display_update(Tn5250Display * This)
    if (This->terminal != NULL) {
       tn5250_terminal_update(This->terminal, This);
       if (This->indicators_dirty) {
-	 tn5250_terminal_update_indicators(This->terminal, This);
+/*	 tn5250_terminal_update_indicators(This->terminal, This);  */
 	 This->indicators_dirty = 0;
       }
    }
@@ -659,7 +652,8 @@ void tn5250_display_interactive_addch(Tn5250Display * This, unsigned char ch)
       ch = toupper(ch);
 
    /* '+' and '-' keys activate field exit/field minus for numeric fields. */
-   if (tn5250_field_is_num_only(field) || tn5250_field_is_signed_num(field)) {
+   if (This->sign_key_hack &&
+      (tn5250_field_is_num_only(field) || tn5250_field_is_signed_num(field))) {
       switch (ch) {
       case '+':
 	 tn5250_display_kf_field_plus(This);
@@ -670,6 +664,7 @@ void tn5250_display_interactive_addch(Tn5250Display * This, unsigned char ch)
 	 return;
       }
    }
+
    /* Make sure this is a valid data character for this field type. */
    if (!tn5250_field_valid_char(field, ch)) {
       TN5250_LOG (("Inhibiting: invalid character for field type.\n"));
@@ -787,11 +782,9 @@ void tn5250_display_field_adjust(Tn5250Display * This, Tn5250Field * field)
    int mand_fill_type;
 
    /* Because of special processing during transmit and other weirdness,
-    * we need to shift signed number fields right regardless.
-    * (num only too?) */
+    * we need to shift signed number fields right regardless. */
    mand_fill_type = tn5250_field_mand_fill_type(field);
-   if (tn5250_field_type(field) == TN5250_FIELD_SIGNED_NUM ||
-	 tn5250_field_type(field) == TN5250_FIELD_NUM_ONLY)
+   if (tn5250_field_type(field) == TN5250_FIELD_SIGNED_NUM) 
       mand_fill_type = TN5250_FIELD_RIGHT_BLANK;
 
    switch (mand_fill_type) {
@@ -1009,6 +1002,22 @@ void tn5250_display_do_key(Tn5250Display *This, int key)
       tn5250_display_kf_dup(This);
       break;
 
+   case K_NEXTWORD:
+      tn5250_display_kf_nextword(This);
+      break;
+
+   case K_PREVWORD:
+      tn5250_display_kf_prevword(This);
+      break;
+
+   case K_FIELDHOME:
+      tn5250_display_kf_fieldhome(This);
+      break;
+
+   case K_NEWLINE:
+      tn5250_display_kf_newline(This);
+      break;
+
    default:
       /* Handle function/command keys. */
       if (key >= K_F1 && key <= K_F24) {
@@ -1189,13 +1198,8 @@ void tn5250_display_kf_field_minus(Tn5250Display * This)
     * signed numeric fields, we change the sign position to a '-'. */
    data = tn5250_display_field_data (This, field);
    if (tn5250_field_type(field) == TN5250_FIELD_NUM_ONLY) {
-      if (data[0] != 0x00 && data[0] != 0x40) {
-	 /* FIXME: Explain why to the user. */
-	 tn5250_display_inhibit(This);
-      } else {
          int i = tn5250_field_length (field) - 1;
-	 if (data[i]&0x20) data[i]&=0xDF;
-      }
+         data[i] = (data[i]&0x0F) | 0xd0;
    } else
       data[tn5250_field_length (field) - 1] = tn5250_char_map_to_remote (This->map, '-');
 
@@ -1647,6 +1651,79 @@ void tn5250_display_kf_delete (Tn5250Display *This)
    }
 }
 
+/****f* lib5250/tn5250_display_kf_prevword
+ * NAME
+ *    tn5250_display_kf_prevword
+ * SYNOPSIS
+ *    tn5250_display_kf_prevword (This);
+ * INPUTS
+ *    Tn5250Display *      This       -
+ * DESCRIPTION
+ *    Move cursor to the last non-blank character.
+ *****/
+void tn5250_display_kf_prevword (Tn5250Display *This)
+{
+      tn5250_dbuffer_prevword(This->display_buffers);
+}
+
+/****f* lib5250/tn5250_display_kf_nextword
+ * NAME
+ *    tn5250_display_kf_nextword
+ * SYNOPSIS
+ *    tn5250_display_kf_nextword (This);
+ * INPUTS
+ *    Tn5250Display *      This       -
+ * DESCRIPTION
+ *    Move cursor to the next non-blank character.
+ *****/
+void tn5250_display_kf_nextword (Tn5250Display *This)
+{
+      tn5250_dbuffer_nextword(This->display_buffers);
+}
+
+/****f* lib5250/tn5250_display_kf_fieldhome
+ * NAME
+ *    tn5250_display_kf_fieldhome
+ * SYNOPSIS
+ *    tn5250_display_kf_fieldhome (This);
+ * INPUTS
+ *    Tn5250Display *      This       -
+ * DESCRIPTION
+ *    Move cursor to the next non-blank character.
+ *****/
+void tn5250_display_kf_fieldhome (Tn5250Display *This)
+{
+   Tn5250Field *field = tn5250_display_current_field(This);
+   if (field != NULL && !tn5250_field_is_bypass(field)) {
+      int y = tn5250_field_start_row (field);
+      int x = tn5250_field_start_col (field);
+      tn5250_display_set_cursor (This, y, x);
+   } else
+      tn5250_display_inhibit(This);
+}
+
+
+/****f* lib5250/tn5250_display_kf_newline
+ * NAME
+ *    tn5250_display_kf_newline
+ * SYNOPSIS
+ *    tn5250_display_kf_newline (This);
+ * INPUTS
+ *    Tn5250Display *      This       - 
+ * DESCRIPTION
+ *    Move cursor to the first field on the next line.
+ *****/
+void tn5250_display_kf_newline (Tn5250Display *This)
+{
+   int y, x;
+
+   y = tn5250_display_cursor_y(This);
+   tn5250_display_set_cursor (This, y, 0);
+   tn5250_dbuffer_down (This->display_buffers);
+   tn5250_display_set_cursor_next_field (This);
+
+}
+
 /****f* lib5250/tn5250_display_save_msg_line
  * NAME
  *    tn5250_display_save_msg_line
