@@ -1,36 +1,24 @@
 /* TN5250
  * Copyright (C) 1997 Michael Madore
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * This file is part of TN5250.
+ *
+ * TN5250 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1, or (at your option)
  * any later version.
  * 
- * This program is distributed in the hope that it will be useful,
+ * TN5250 is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this software; see the file COPYING.  If not, write to
  * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307 USA
  * 
- * As a special exception, the copyright holder gives permission
- * for additional uses of the text contained in its release of TN5250.
- * 
- * The exception is that, if you link the TN5250 library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the TN5250 library code into it.
- * 
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- * 
- * If you write modifications of your own for TN5250, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice. */
+ */
 #include "tn5250-private.h"
 
 /****f* lib5250/tn5250_field_new
@@ -101,6 +89,7 @@ void tn5250_field_destroy(Tn5250Field * This)
  * DESCRIPTION
  *    DOCUMENT ME!!!
  *****/
+#ifndef NDEBUG
 void tn5250_field_dump(Tn5250Field * This)
 {
    int curchar;
@@ -126,6 +115,7 @@ void tn5250_field_dump(Tn5250Field * This)
    
    TN5250_LOG(("\n"));
 }
+#endif
 
 /****f* lib5250/tn5250_field_hit_test
  * NAME
@@ -357,10 +347,8 @@ int tn5250_field_valid_char (Tn5250Field *field, int ch)
 
    case TN5250_FIELD_NUM_ONLY:
       return (isdigit (ch) ||
-	  ch == '+' ||
 	  ch == ',' ||
 	  ch == '.' ||
-	  ch == '-' ||
 	  ch == ' ');
 
    case TN5250_FIELD_KATA_SHIFT:
@@ -375,9 +363,7 @@ int tn5250_field_valid_char (Tn5250Field *field, int ch)
       return 1;
 
    case TN5250_FIELD_SIGNED_NUM:
-      return (isdigit(ch) ||
-	  ch == '+' ||
-	  ch == '-');
+      return (isdigit(ch));
    }
    return 0;
 }
