@@ -1,36 +1,24 @@
 /* TN5250 - An implementation of the 5250 telnet protocol.
  * Copyright (C) 1997 Michael Madore
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * This file is part of TN5250.
+ *
+ * TN5250 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1, or (at your option)
  * any later version.
  * 
- * This program is distributed in the hope that it will be useful,
+ * TN5250 is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this software; see the file COPYING.  If not, write to
  * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307 USA
  * 
- * As a special exception, the copyright holder gives permission
- * for additional uses of the text contained in its release of TN5250.
- * 
- * The exception is that, if you link the TN5250 library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the TN5250 library code into it.
- * 
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- * 
- * If you write modifications of your own for TN5250, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice. */
+ */
 
 #include "tn5250-private.h"
 
@@ -528,6 +516,8 @@ static void tn5250_session_process_stream(Tn5250Session * This)
       if (cur_command != ESC) {
 	 TN5250_LOG (("cur_command != ESC; cur_pos = %d\n",
 		  This->record->cur_pos));
+         TN5250_LOG (("Ignoring record!\n"));
+         return;
 	 TN5250_ASSERT(0);
       }
       cur_command = tn5250_record_get_byte(This->record);
@@ -573,6 +563,9 @@ static void tn5250_session_process_stream(Tn5250Session * This)
       case CMD_ROLL:
 	 tn5250_session_roll(This);
 	 break;
+      case 0x0a:
+         TN5250_LOG (("Ignoring record!\n"));
+	 break;
       default:
 	 TN5250_LOG(("Error: Unknown command 0x%02X.\n", cur_command));
 
@@ -1308,6 +1301,7 @@ static void tn5250_session_start_of_header(Tn5250Session * This)
    TN5250_LOG (("StartOfHeader: entered.\n"));
    
    tn5250_dbuffer_clear_table(tn5250_display_dbuffer(This->display));
+   tn5250_display_clear_pending_insert(This->display);
    tn5250_display_indicator_set(This->display, TN5250_DISPLAY_IND_X_SYSTEM);
 
    n = tn5250_record_get_byte (This->record);
