@@ -1,40 +1,31 @@
 /* TN5250 - An implementation of the 5250 telnet protocol.
  * Copyright (C) 1997 Michael Madore
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * This file is part of TN5250.
+ *
+ * TN5250 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1, or (at your option)
  * any later version.
  * 
- * This program is distributed in the hope that it will be useful,
+ * TN5250 is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this software; see the file COPYING.  If not, write to
  * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307 USA
  * 
- * As a special exception, the copyright holder gives permission
- * for additional uses of the text contained in its release of TN5250.
- * 
- * The exception is that, if you link the TN5250 library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the TN5250 library code into it.
- * 
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- * 
- * If you write modifications of your own for TN5250, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice. */
+ */
 #include "tn5250-private.h"
 
 /* External declarations of initializers for each type of stream. */
 extern int tn5250_telnet_stream_init (Tn5250Stream *This);
+#ifdef HAVE_LIBSSL
+extern int tn5250_ssl_stream_init (Tn5250Stream *This);
+#endif
 #ifndef NDEBUG
 extern int tn5250_debug_stream_init (Tn5250Stream *This);
 #endif
@@ -54,6 +45,11 @@ static Tn5250StreamType stream_types[] = {
 #endif
    { "telnet:", tn5250_telnet_stream_init },
    { "tn5250:", tn5250_telnet_stream_init },
+#ifdef HAVE_LIBSSL
+   { "ssl:", tn5250_ssl_stream_init },
+   { "telnet-ssl:", tn5250_ssl_stream_init },
+   { "telnets:", tn5250_ssl_stream_init },
+#endif
    { NULL, NULL }
 };
 
@@ -96,7 +92,7 @@ static void streamInit(Tn5250Stream *This, long timeout)
  *    The next protocol to add is SNA.  This will allow the emulator to use 
  *    APPC (LU 6.2) to establish a session with the AS/400.
  *****/
-Tn5250Stream *tn5250_stream_open (const char *to)
+Tn5250Stream *tn5250_stream_open (const char *to, Tn5250Config *config)
 {
    Tn5250Stream *This = tn5250_new(Tn5250Stream, 1);
    Tn5250StreamType *iter;
@@ -107,6 +103,9 @@ Tn5250Stream *tn5250_stream_open (const char *to)
 
       streamInit(This, 0);
 
+      if (config != NULL)
+           tn5250_stream_config(This, config);
+
       /* Figure out the stream type. */
       iter = stream_types;
       while (iter->prefix != NULL) {
@@ -154,7 +153,7 @@ Tn5250Stream *tn5250_stream_open (const char *to)
  * DESCRIPTION
  *    DOCUMENT ME!!!
  *****/
-Tn5250Stream *tn5250_stream_host (int masterfd, long timeout)
+Tn5250Stream *tn5250_stream_host (SOCKET_TYPE masterfd, long timeout)
 {
    Tn5250Stream *This = tn5250_new(Tn5250Stream, 1);
    Tn5250StreamType *iter;
