@@ -1,36 +1,24 @@
 /* TN5250 - An implementation of the 5250 telnet protocol.
  * Copyright (C) 1997 Michael Madore
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * This file is part of TN5250.
+ *
+ * TN5250 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1, or (at your option)
  * any later version.
  * 
- * This program is distributed in the hope that it will be useful,
+ * TN5250 is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this software; see the file COPYING.  If not, write to
  * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307 USA
  * 
- * As a special exception, the copyright holder gives permission
- * for additional uses of the text contained in its release of TN5250.
- * 
- * The exception is that, if you link the TN5250 library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the TN5250 library code into it.
- * 
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- * 
- * If you write modifications of your own for TN5250, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice. */
+ */
 #ifndef STREAM5250_H
 #define STREAM5250_H
 
@@ -38,6 +26,10 @@
 # include <winsock.h>  /* Need for SOCKET type.  GJS 3/3/2000 */
 #endif 
 
+#ifdef HAVE_LIBSSL
+#include <openssl/ssl.h>
+#endif
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -87,6 +79,12 @@ struct _Tn5250Stream {
    int state;
   long msec_wait;
 
+#ifdef HAVE_LIBSSL
+  SSL *ssl_handle;
+  SSL_CTX *ssl_context;
+  void *userdata;
+#endif
+
 #ifndef NDEBUG
    FILE *debugfile;
 #endif
@@ -95,7 +93,7 @@ struct _Tn5250Stream {
 typedef struct _Tn5250Stream Tn5250Stream;
 /******/
 
-extern Tn5250Stream /*@only@*/ /*@null@*/ *tn5250_stream_open (const char *to);
+extern Tn5250Stream /*@only@*/ /*@null@*/ *tn5250_stream_open (const char *to, struct _Tn5250Config *config);
 extern int tn5250_stream_config (Tn5250Stream *This, struct _Tn5250Config *config);
 extern void tn5250_stream_destroy(Tn5250Stream /*@only@*/ * This);
 extern Tn5250Record /*@only@*/ *tn5250_stream_get_record(Tn5250Stream * This);
