@@ -1,39 +1,27 @@
 /* TN5250 - An implementation of the 5250 telnet protocol.
  * Copyright (C) 1997 Michael Madore
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * This file is part of TN5250.
+ *
+ * TN5250 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1, or (at your option)
  * any later version.
  * 
- * This program is distributed in the hope that it will be useful,
+ * TN5250 is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this software; see the file COPYING.  If not, write to
  * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307 USA
  * 
- * As a special exception, the copyright holder gives permission
- * for additional uses of the text contained in its release of TN5250.
- * 
- * The exception is that, if you link the TN5250 library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the TN5250 library code into it.
- * 
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- * 
- * If you write modifications of your own for TN5250, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice. */
+ */
 #include "tn5250-private.h"
 
-static int telnet_stream_get_next(Tn5250Stream * This);
+static int telnet_stream_get_next(Tn5250Stream * This, unsigned char *buf, int size);
 static void telnet_stream_do_verb(Tn5250Stream * This, unsigned char verb, unsigned char what);
 static int telnet_stream_host_verb(Tn5250Stream * This, unsigned char verb,
 	unsigned char what);
@@ -94,7 +82,9 @@ static void telnet_stream_send_packet(Tn5250Stream * This, int length,
 #define RECV_EOR	4
 #define SEND_EOR	8
 
+#ifndef HAVE_UCHAR
 typedef unsigned char UCHAR;
+#endif
 
 static UCHAR hostInitStr[] = {IAC,DO,NEW_ENVIRON,IAC,DO,TERMINAL_TYPE};
 static UCHAR hostDoEOR[] = {IAC,DO,END_OF_RECORD};
@@ -388,7 +378,7 @@ static int telnet_stream_connect(Tn5250Stream * This, const char *to)
  * DESCRIPTION
  *    Accepts a connection from the client.
  *****/
-static int telnet_stream_accept(Tn5250Stream * This, int masterfd)
+static int telnet_stream_accept(Tn5250Stream * This, SOCKET_TYPE masterfd)
 {
    int i, len, retCode;
    struct sockaddr_in serv_addr;
@@ -483,14 +473,17 @@ static void telnet_stream_destroy(Tn5250Stream *This)
  * NAME
  *    telnet_stream_get_next
  * SYNOPSIS
- *    ret = telnet_stream_get_next (This);
+ *    ret = telnet_stream_get_next (This, buf, size);
  * INPUTS
  *    Tn5250Stream *       This       - 
+ *    unsigned char *      buf        -
+ *    int                  size       -
  * DESCRIPTION
- *    Gets the next byte from the socket or returns -1 if no data is
- *    currently available on the socket or -2 if we have been disconnected.
+ *    Gets the next buffer of data from the socket.  The
+ *    return value is the length of the data received,
+ *    or -1 for no data to receive, or -2 if disconnected
  *****/
-static int telnet_stream_get_next(Tn5250Stream * This)
+static int telnet_stream_get_next(Tn5250Stream * This, unsigned char *buf, int size)
 {
    unsigned char curchar;
    int rc;
@@ -505,10 +498,10 @@ static int telnet_stream_get_next(Tn5250Stream * This)
    if (!FD_ISSET(This->sockfd, &fdr))
       return -1;		/* No data on socket. */
 
-   rc = TN_RECV(This->sockfd, (char *) &curchar, 1, 0);
+   rc = TN_RECV(This->sockfd, buf, size, 0);
    if (WAS_ERROR_RET(rc)) {
       if (LAST_ERROR != ERR_AGAIN && LAST_ERROR != ERR_INTR) {
-	 printf("Error reading from socket: %s\n", strerror(LAST_ERROR));
+	 TN5250_LOG(("Error reading from socket: %s\n", strerror(LAST_ERROR)));
 	 return -2;
       } else
 	 return -1;
@@ -518,7 +511,7 @@ static int telnet_stream_get_next(Tn5250Stream * This)
    if (rc == 0)
       return -2;
 
-   return (int) curchar;
+   return rc;
 }
 
 static int sendWill(SOCKET_TYPE sock, unsigned char what)
@@ -782,7 +775,7 @@ static void telnet_stream_sb(Tn5250Stream * This, unsigned char *sb_buf, int sb_
 	 printf("Error writing to socket: %s\n", strerror(LAST_ERROR));
 	 exit(5);
       }
-      TN5250_LOG(("SentSB:<IAC><SB><TERMTYPE><IS>%s<IAC><SE>\n"));
+      TN5250_LOG(("SentSB:<IAC><SB><TERMTYPE><IS>%s<IAC><SE>\n", termtype));
 
       This->status = This->status | TERMINAL;
    } else if (sb_buf[0] == NEW_ENVIRON) {
@@ -832,18 +825,28 @@ static void telnet_stream_sb(Tn5250Stream * This, unsigned char *sb_buf, int sb_
  *    is waiting on the socket or -2 if disconnected, or -END_OF_RECORD if a 
  *    telnet EOR escape sequence was encountered.
  *****/
+#define TN5250_RBSIZE 8192
 static int telnet_stream_get_byte(Tn5250Stream * This)
 {
    int temp;
    unsigned char verb;
+   static unsigned char rcvbuf[TN5250_RBSIZE];
+   static int rcvbufpos = 0;
+   static int rcvbuflen = -1;
+   
 
    do {
       if (This->state == TN5250_STREAM_STATE_NO_DATA)
 	 This->state = TN5250_STREAM_STATE_DATA;
 
-      temp = telnet_stream_get_next(This);
-      if (temp < 0)
-	 return temp;
+      rcvbufpos ++;
+      if (rcvbufpos >= rcvbuflen) {
+          rcvbufpos = 0;
+          rcvbuflen = telnet_stream_get_next(This, rcvbuf, TN5250_RBSIZE);
+          if (rcvbuflen<0) 
+               return rcvbuflen;
+      }
+      temp = rcvbuf[rcvbufpos];
 
       switch (This->state) {
       case TN5250_STREAM_STATE_DATA:
@@ -1099,7 +1102,8 @@ int telnet_stream_handle_receive(Tn5250Stream * This)
 
       if (c == -END_OF_RECORD && This->current_record != NULL) {
 	 /* End of current packet. */
-	 tn5250_record_dump(This->current_record);
+         if (tn5250_logfile!=NULL) 
+             tn5250_record_dump(This->current_record);
 	 This->records = tn5250_record_list_add(This->records, This->current_record);
 	 This->current_record = NULL;
 	 This->record_count++;
