@@ -1,36 +1,24 @@
 /* TN5250 - An implementation of the 5250 telnet protocol.
  * Copyright (C) 1997 Michael Madore
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * This file is part of TN5250.
+ *
+ * TN5250 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1, or (at your option)
  * any later version.
  * 
- * This program is distributed in the hope that it will be useful,
+ * TN5250 is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this software; see the file COPYING.  If not, write to
  * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307 USA
  * 
- * As a special exception, the copyright holder gives permission
- * for additional uses of the text contained in its release of TN5250.
- * 
- * The exception is that, if you link the TN5250 library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the TN5250 library code into it.
- * 
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- * 
- * If you write modifications of your own for TN5250, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice. */
+ */
 #ifndef TERMINAL_H
 #define TERMINAL_H
 
@@ -57,7 +45,7 @@ extern "C" {
 #define K_FIRST_SPECIAL	0400	/* Coincides with first Curses key. */
 
 #define K_ENTER		0x0d
-#define K_NEWLINE	0x0a
+#define K_NEWLINE	0x200
 #define K_TAB		0x09
 #define K_BACKTAB	0541
 #define K_F1		(0410+1)
@@ -109,6 +97,11 @@ extern "C" {
 #define K_DUPLICATE	0524	/* curses KEY_STAB (set tab - good as any) */
 #define K_FIELDMINUS	0526	/* curses KEY_CATAB (clear all tabs - g.a.a.) */
 #define K_FIELDPLUS     0520    /* curses KEY_SF */
+#define K_PREVWORD	0611	/* curses KEY_SLEFT (as good as any) */
+#define K_NEXTWORD	0622	/* curses KEY_SRIGHT (as good as any) */
+#define K_FIELDHOME	0607	/* curses KEY_SHOME (as good as any) */
+#define K_COPY_TEXT     0xfffd  /* no similar curses key */
+#define K_PASTE_TEXT    0xfffe  /* no similar curses key */
 #define K_UNKNOW	0xffff
 
 struct _Tn5250Display;
