@@ -1,17 +1,19 @@
 /*
  * Copyright (C) 1997 Michael Madore
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * This file is part of TN5250.
+ *
+ * TN5250 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * TN5250 is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
@@ -81,13 +83,10 @@ int main(int argc, char *argv[])
       tn5250_log_open (tn5250_config_get (config, "trace"));
 #endif
 
-   stream = tn5250_stream_open (tn5250_config_get (config, "host"));
+   stream = tn5250_stream_open (tn5250_config_get (config, "host"), config);
    if (stream == NULL)
       goto bomb_out;
 
-   if (tn5250_stream_config (stream, config) == -1)
-      goto bomb_out;
-
    display = tn5250_display_new ();
    if (tn5250_display_config (display, config) == -1)
       goto bomb_out;
@@ -111,6 +110,7 @@ int main(int argc, char *argv[])
             tn5250_config_get (config, "font_132")
             );
    }
+   tn5250_curses_terminal_load_colorlist(config);
 #endif
 #ifdef USE_SLANG
    term = tn5250_slang_terminal_new();
@@ -176,8 +176,13 @@ static void syntax()
 
    printf ("tn5250 - TCP/IP 5250 emulator\n\
 Syntax:\n\
-  tn5250 [options] HOST[:PORT]\n\
-\n\
+  tn5250 [options] HOST[:PORT]\n");
+#ifdef HAVE_LIBSSL
+   printf("\
+   To connect using ssl prefix HOST with 'ssl:'.  Example:
+      tn5250 +ssl_verify_server ssl:as400.example.com\n");
+#endif
+   printf ("\n\
 Options:\n\
    map=NAME                Character map (default is '37'):");
    m = tn5250_transmaps;
@@ -193,7 +198,16 @@ Options:\n\
    printf ("\
    trace=FILE              Log session to FILE.\n");
 #endif
-   printf ("
+#ifdef HAVE_LIBSSL
+   printf ("\
+   +/-ssl_verify_server    Verify/don't verify the server's SSL certificate\n\
+   ssl_ca_file=FILE        Use certificate authority (CA) certs from FILE\n\
+   ssl_cert_file=FILE      File containing SSL certificate in PEM format to\n\
+                           use if the AS/400 requires client authentication.\n\
+   ssl_pem_pass=PHRASE     Passphrase to use when decrypting a PEM private\n\
+                           key.  Used in conjunction with ssl_cert_file\n");
+#endif
+   printf ("\
    +/-underscores          Use/don't use underscores instead of underline\n\
                            attribute.\n\
    +/-ruler                Draw a ruler pointing to the cursor position\n\
