@@ -1,39 +1,34 @@
 /* TN5250 - An implementation of the 5250 telnet protocol.
  * Copyright (C) 1997 Michael Madore
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * This file is part of TN5250.
+ *
+ * TN5250 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1, or (at your option)
  * any later version.
  * 
- * This program is distributed in the hope that it will be useful,
+ * TN5250 is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this software; see the file COPYING.  If not, write to
  * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307 USA
  * 
- * As a special exception, the copyright holder gives permission
- * for additional uses of the text contained in its release of TN5250.
- * 
- * The exception is that, if you link the TN5250 library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the TN5250 library code into it.
- * 
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- * 
- * If you write modifications of your own for TN5250, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice. */
+ */
 #include "tn5250-private.h"
 #include "transmaps.h"
 
+static char mapfix[256];
+static char mapfix2[256];
+static char mapfix3[256];
+static char mapfix4[256];
+
+#ifndef WIN32
+
 /****f* lp5250d/tn5250_closeall
  * NAME
  *    tn5250_closeall
@@ -136,6 +131,8 @@ int tn5250_daemon(int nochdir, int noclose, int ignsigcld)
     return 0;
 }
 
+#endif  /* ifndef WIN32 */
+
 int 
 tn5250_make_socket (unsigned short int port)
 {
@@ -148,7 +145,9 @@ tn5250_make_socket (unsigned short int port)
   sock = socket (PF_INET, SOCK_STREAM, 0);
   if (sock < 0)
     {
+#ifndef WIN32
       syslog(LOG_INFO, "socket: %s\n", strerror(errno));
+#endif
       exit (EXIT_FAILURE);
     }
 
@@ -156,11 +155,13 @@ tn5250_make_socket (unsigned short int port)
   name.sin_family = AF_INET;
   name.sin_port = htons (port);
   name.sin_addr.s_addr = htonl (INADDR_ANY);
-  setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on));
+  setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (const char *)&on, sizeof(on));
   TN_IOCTL(sock, FIONBIO, &ioctlarg);
   if (bind (sock, (struct sockaddr *) &name, sizeof (name)) < 0)
     {
+#ifndef WIN32
       syslog(LOG_INFO, "bind: %s\n", strerror(errno));
+#endif
       exit (EXIT_FAILURE);
     }
 
@@ -221,6 +222,76 @@ Tn5250Char tn5250_char_map_to_local(Tn5250CharMap *map, Tn5250Char ebcdic)
 Tn5250CharMap *tn5250_char_map_new (const char *map)
 {
    Tn5250CharMap *t;
+
+/* XXX: HACK: These characters were reported wrong in transmaps.h.
+        Since that's a generated file, I'm overriding them here -SCK */
+
+   TN5250_LOG(("tn5250_char_map_new: map = \"%s\"\n", map));
+
+   if (!strcmp(map, "870") || !strcmp(map, "win870")) {
+
+      TN5250_LOG(("tn5250_char_map_new: Installing 870 workaround\n"));
+
+      memcpy(mapfix, windows_1250_to_ibm870, sizeof(mapfix));
+      memcpy(mapfix2, ibm870_to_windows_1250, sizeof(mapfix2));
+      memcpy(mapfix3, iso_8859_2_to_ibm870, sizeof(mapfix3));
+      memcpy(mapfix4, ibm870_to_iso_8859_2, sizeof(mapfix4));
+
+      mapfix[142] = 184;
+      mapfix[143] = 185;
+      mapfix[158] = 182;
+      mapfix[159] = 183;
+      mapfix[163] = 186;
+      mapfix[202] = 114;
+      mapfix[234] = 82;
+
+      mapfix2[82] = 234;
+      mapfix2[114] = 202;
+      mapfix2[182] = 158;
+      mapfix2[183] = 159;
+      mapfix2[184] = 142;
+      mapfix2[185] = 143;
+      mapfix2[186] = 163;
+
+      mapfix3[163] = 186;
+      mapfix3[172] = 185;
+      mapfix3[188] = 183;
+      mapfix3[202] = 114;
+      mapfix3[234] = 82;
+
+      mapfix4[82] = 234;
+      mapfix4[114] = 202;
+      mapfix4[183] = 188;
+      mapfix4[185] = 172;
+      mapfix4[186] = 163;
+
+      for (t = tn5250_transmaps;  t->name; t++) {
+          if (!strcmp(t->name, "win870")) {
+               t->to_remote_map = (const char *)mapfix;
+               t->to_local_map = (const char *)mapfix2;
+               TN5250_LOG(("Workaround installed for map \"win870\"\n"));
+          }
+          else if (!strcmp(t->name, "870")) {
+               t->to_remote_map = (const char *)mapfix3;
+               t->to_local_map = (const char *)mapfix4;
+               TN5250_LOG(("Workaround installed for map \"870\"\n"));
+          }
+      }
+   }
+   
+   /* Under Windows, we'll try the "winXXX" maps first, then fall back 
+      to the standard (unix) versions */
+#ifdef WIN32
+   {
+      char winmap[10];
+      _snprintf(winmap, sizeof(winmap)-1, "win%s", map);
+      for (t = tn5250_transmaps; t->name; t++) {
+         if (strcmp(t->name, winmap) == 0)
+   	 return t;
+      }
+   }
+#endif
+
    for (t = tn5250_transmaps; t->name; t++) {
       if (strcmp(t->name, map) == 0)
 	 return t;
@@ -382,5 +453,116 @@ void tn5250_log_assert(int val, char const *expr, char const *file, int line)
    }
 }
 
+
+/****f* lib5250/tn5250_parse_color
+ * NAME
+ *    tn5250_parse_color
+ * SYNOPSIS
+ *    tn5250_parse_color (config, "green", &red, &green, &blue);
+ * INPUTS
+ *    Tn5250Config *       config     -
+ *    const char   *       colorname  -
+ *    int          *       red        - 
+ *    int          *       green      - 
+ *    int          *       blue       - 
+ * DESCRIPTION
+ *    This loads a color from the TN5250 config object, and then
+ *    parses it into it's red, green, blue components.
+ *****/
+int tn5250_parse_color(Tn5250Config *config, const char *colorname, 
+                        int *red, int *green, int *blue) {
+
+    const char *p;
+    char colorspec[16];
+    int r, g, b;
+
+    if ((p=tn5250_config_get(config, colorname)) == NULL) {
+        return -1;
+    }
+
+    strncpy(colorspec, p, sizeof(colorspec));
+    colorspec[sizeof(colorspec)-1] = '\0';
+
+    if (*colorspec != '#') {
+          if (!strcasecmp(colorspec, "white")) {
+               r = 255; g = 255; b = 255;
+          } else if (!strcasecmp(colorspec, "yellow")) {
+               r = 255; g = 255; b = 0;
+          } else if (!strcasecmp(colorspec, "lightmagenta")) {
+               r = 255; g = 0;   b = 255;
+          } else if (!strcasecmp(colorspec, "lightred")) {
+               r = 255; g = 0;   b = 0;
+          } else if (!strcasecmp(colorspec, "lightcyan")) {
+               r = 0;   g = 255; b = 255;
+          } else if (!strcasecmp(colorspec, "lightgreen")) {
+               r = 0;   g = 255; b = 0;
+          } else if (!strcasecmp(colorspec, "lightblue")) {
+               r = 0;   g = 0;   b = 255;
+          } else if (!strcasecmp(colorspec, "lightgray")) {
+               r = 192; g = 192; b = 192;
+          } else if (!strcasecmp(colorspec, "gray")) {
+               r = 128; g = 128; b = 128;
+          } else if (!strcasecmp(colorspec, "brown")) {
+               r = 128; g = 128; b = 0;
+          } else if (!strcasecmp(colorspec, "red")) {
+               r = 128; g = 0;   b = 0;
+          } else if (!strcasecmp(colorspec, "cyan")) {
+               r = 0;   g = 128; b = 128;
+          } else if (!strcasecmp(colorspec, "green")) {
+               r = 0;   g = 128; b = 0;
+          } else if (!strcasecmp(colorspec, "blue")) {
+               r = 0;   g = 0;   b = 128;
+          } else if (!strcasecmp(colorspec, "black")) {
+               r = 0;   g = 0;   b = 0;
+          }
+    }
+    else {
+         if (strlen(colorspec) != 7)
+              return -1;
+         if (sscanf(&colorspec[1], "%02x%02x%02x", &r, &g, &b)!=3) 
+              return -1;
+    }
+
+    *red = r;
+    *green = g;
+    *blue = b;
+    return 0;
+}
 #endif				/* NDEBUG */
 
+
+/****f* lib5250/tn5250_setenv
+ * NAME
+ *    tn5250_setenv
+ * SYNOPSIS
+ *    tn5250_setenv ("TN5250_CCSID", "37", 1);
+ * INPUTS
+ *    const char   *       name       -
+ *    const char   *       value      - 
+ *    int                  overwrite  - 
+ * DESCRIPTION
+ *    This works just like setenv(3), but setenv(3) doesn't
+ *    exist on all systems. 
+ *****/
+int tn5250_setenv(const char *name, const char *value, int overwrite) {
+
+     char *strval;
+     int ret;
+
+     if (!overwrite) 
+         if (getenv(name)!=NULL) return 0;
+
+     strval = malloc(strlen(name)+strlen(value)+2);
+     TN5250_ASSERT(strval!=NULL);
+
+     strcpy(strval, name);
+     strcat(strval, "=");
+     strcat(strval, value);
+
+     ret = putenv(strval);
+
+     /* free(strval)   on some systems, it continues to use our memory,
+                       so we should not free it...                    */
+
+     return ret;
+}
