@@ -1,36 +1,24 @@
 /* TN5250 - An implementation of the 5250 telnet protocol.
  * Copyright (C) 1997 Michael Madore
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * This file is part of TN5250.
+ *
+ * TN5250 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1, or (at your option)
  * any later version.
  * 
- * This program is distributed in the hope that it will be useful,
+ * TN5250 is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this software; see the file COPYING.  If not, write to
  * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307 USA
  * 
- * As a special exception, the copyright holder gives permission
- * for additional uses of the text contained in its release of TN5250.
- * 
- * The exception is that, if you link the TN5250 library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the TN5250 library code into it.
- * 
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- * 
- * If you write modifications of your own for TN5250, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice. */
+ */
 #ifndef UTILITY_H
 #define UTILITY_H
 
@@ -91,6 +79,7 @@ Tn5250Char tn5250_char_map_to_local(Tn5250CharMap *This, Tn5250Char ebcdic);
 
 int tn5250_char_map_printable_p(Tn5250CharMap *This, Tn5250Char data);
 int tn5250_char_map_attribute_p(Tn5250CharMap *This, Tn5250Char data);
+int tn5250_setenv(const char *name, const char *value, int overwrite);
 
 /* Idea shamelessly stolen from GTK+ */
 #define tn5250_new(type,count) (type *)malloc (sizeof (type) * (count))
@@ -108,8 +97,13 @@ void tn5250_log_assert(int val, char const *expr, char const *file, int line);
 extern FILE * tn5250_logfile;
 #else
 #define TN5250_LOG(args)
+#define TN5250_ASSERT(expr)
 #endif
 
+#include "conf.h"
+
+int tn5250_parse_color(Tn5250Config *config, const char *colorname, 
+                        int *r, int *g, int *b);
 #ifdef __cplusplus
 }
 #endif
